/*-
 * Copyright (c) 2005 M. Warner Losh.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $FreeBSD: src/sys/arm/at91/at91_streg.h,v 1.1 2006/02/04 23:32:13 imp Exp $ */

#ifndef ARM_AT91_AT91TCREG_H
#define ARM_AT91_AT91TCREG_H

/* For the entire device */
#define TC_TC0_OFFSET	0
#define TC_TC1_OFFSET	0x40
#define TC_TC2_OFFSET	0x80
#define	TC_BCR		0xc0		/* TC Block Control Register */
#define		TC_BCR_SYNC	1
#define	TC_BMR		0xc4		/* TC Block Mode Register */
#define		TC_BMR_XC0_TCLK0	0
#define		TC_BMR_XC0_NONE		1
#define		TC_BMR_XC0_TIOA1	2
#define		TC_BMR_XC0_TIOA2	3
#define		TC_BMR_XC1_TCLK1	(0 << 2)
#define		TC_BMR_XC1_NONE		(1 << 2)
#define		TC_BMR_XC1_TIOA0	(2 << 2)
#define		TC_BMR_XC1_TIOA2	(3 << 2)
#define		TC_BMR_XC2_TCLK2	(0 << 4)
#define		TC_BMR_XC2_NONE		(1 << 4)
#define		TC_BMR_XC2_TIOA0	(2 << 4)
#define		TC_BMR_XC2_TIOA1	(3 << 4)

/* For each TC */

#define	TC_CCR		0x0		/* CCR: Chanel Control Register */
#define		TC_CCR_CLKEN	1		/* Clock enable */
#define		TC_CCR_CLKDIS	2		/* Clock disable */
#define		TC_CCR_SWTRG	4		/* Software Trigger */
#define	TC_CMR		0x4		/* CMR: Channel Mode Register */
	/* WAVE=0 */
#define		TC_CMR_TCCLKS_MASK 0x7		/* Clock Selection */
#define		TC_CMR_TIMER_CLOCK1	0
#define		TC_CMR_TIMER_CLOCK2	1
#define		TC_CMR_TIMER_CLOCK3	2
#define		TC_CMR_TIMER_CLOCK4	3
#define		TC_CMR_TIMER_CLOCK5	4
#define		TC_CMR_XC0		5
#define		TC_CMR_XC1		6
#define		TC_CMR_XC2		7
#define		TC_CMR_CLKI	0x8		/* Clock Invert */
#define		TC_CMR_BURST_MASK  0x30		/* Burst signal selection */
#define		TC_CMR_BURST_NONE	0
#define		TC_CMR_BURST_XC0	0x10
#define		TC_CMR_BURST_XC1	0x20
#define		TC_CMR_BURST_XC2	0x30
#define		TC_CMR_LDBSTOP	0x40	/* Clock Counter stop @ RB Loading */
#define		TC_CMR_LDBDIS	0x80	/* Clock counter dis  @ RB Loading */
#define		TC_CMR_ETRGEDG_MASK 0x300	/* External Trigger Edge sel */
#define		TC_CMR_ETRGEDG_NONE	0
#define		TC_CMR_ETRGEDG_RISE	0x100
#define		TC_CMR_ETRGEDG_FALL	0x200
#define		TC_CMR_ETRGEDG_BOTH	0x300
#define		TC_CMR_ABETRG	0x400		/* TIOA or TIOB Trigger */
#define		TC_CMR_CPCTRG	0x4000		/* Compare Trigger Enable */
#define		TC_CMR_WAVE	0x8000		/* WAVE */
#define		TC_CMR_LDRA_MASK	0x30000
#define		TC_CMR_LDRA_NONE	0
#define		TC_CMR_LDRA_RISE	0x10000
#define		TC_CMR_LDRA_FALL	0x20000
#define		TC_CMR_LDRA_BOTH	0x30000
#define		TC_CMR_LDRB_MASK	0xc0000
#define		TC_CMR_LDRB_NONE	0
#define		TC_CMR_LDRB_RISE	0x40000
#define		TC_CMR_LDRB_FALL	0x80000
#define		TC_CMR_LDRB_BOTH	0xc0000
	/* WAVE=1 */ /* XXX Omitted XXX */
#define	TC_CV		0x10		/* CV: Current Value */
#define	TC_RA		0x14		/* RA: Register A */
#define	TC_RB		0x18		/* RB: Register B */
#define	TC_RC		0x1c		/* RC: Register C */
#define	TC_SR		0x20		/* SR: Status Register */
#define		TC_SR_COVFS	0x00001		/* Counter Overflow Status */
#define		TC_SR_LOVRS	0x00002		/* Load Overrun Status */
#define		TC_SR_CPAS	0x00004		/* RA Compare Status */
#define		TC_SR_CPBS	0x00008		/* RB Compare Status */
#define		TC_SR_CPCS	0x00010		/* RC Compare Status */
#define		TC_SR_LDRAS	0x00020		/* RA Loading Status */
#define		TC_SR_LDRBS	0x00040		/* RB Loading Status */
#define		TC_SR_ETRGS	0x00080		/* External Trigger Status */
#define		TC_SR_CLKSTA	0x10000		/* Clock Enabling Status */
#define		TC_SR_MTIOA	0x20000		/* TIOA Mirror */
#define		TC_SR_MTIOB	0x40000		/* TIOB Mirror */

#define	TC_IER		0x24		/* IER: Interrupt Enable Register */
#define	TC_IDR		0x28		/* IDR: Interrupt Disable Register */
#define	TC_IMR		0x2c		/* IMR: Interrupt Mask Register */

#endif /* ARM_AT91_AT91TCREG_H */
