/* $NetBSD: ep93xx_intr.c,v 1.3 2005/01/05 04:53:50 joff Exp $ */

/*
 * Copyright (c) 2002 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Jesse Off
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Ichiro FUKUHARA and Naoto Shimazaki.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by the NetBSD
 *        Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $NetBSD: ep93xx_intr.c,v 1.3 2005/01/05 04:53:50 joff Exp $
 *
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD");

/*
 * Interrupt support for the Cirrus Logic EP93XX
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/module.h>
#include <sys/bus.h>
#define _ARM32_BUS_DMA_PRIVATE
#include <sys/rman.h>

#include <machine/bus.h>
#include <machine/intr.h>

#include <machine/cpufunc.h>

#include <arm/ep93xx/ep93xxreg.h> 
#include <arm/ep93xx/ep93xxvar.h> 
#include <arm/ep93xx/ep93xx.h>

#include <arm/ep93xx/tsarmreg.h> /* XXX */

/* Software copy of the IRQs we have enabled. */
__volatile u_int32_t vic1_intr_enabled;
__volatile u_int32_t vic2_intr_enabled;

#define VIC1REG(reg)	*((volatile u_int32_t*) (EP93XX_AHB_VBASE + \
	EP93XX_AHB_VIC1 + (reg)))
#define VIC2REG(reg)	*((volatile u_int32_t*) (EP93XX_AHB_VBASE + \
	EP93XX_AHB_VIC2 + (reg)))

void
arm_unmask_irq(int irq)
{
	KASSERT(irq < VIC_NIRQ * 2, ("irq %d is greater than max", irq));

	if (irq < VIC_NIRQ) {
		vic1_intr_enabled |= (1U << irq);
		VIC1REG(EP93XX_VIC_IntEnable) = (1U << irq);
	} else {
		vic2_intr_enabled |= (1U << (irq - VIC_NIRQ));
		VIC2REG(EP93XX_VIC_IntEnable) = 
		    (1U << (irq - VIC_NIRQ));
	}
}

void
arm_mask_irq(int irq)
{
	KASSERT(irq < VIC_NIRQ * 2, ("irq %d is greater than max", irq));

	if (irq < VIC_NIRQ) {
		vic1_intr_enabled &= ~(1U << irq);
		VIC1REG(EP93XX_VIC_IntEnClear) = (1U << irq);
	} else {
		vic2_intr_enabled &= ~(1U << (irq - VIC_NIRQ));
		VIC2REG(EP93XX_VIC_IntEnClear) = 
		    (1U << (irq - VIC_NIRQ));
	}
}

int
arm_get_next_irq()
{
	int reg;

	if ((reg = VIC1REG(EP93XX_VIC_IRQStatus) & vic1_intr_enabled)) {
		return ffs(reg) - 1;
	} else if ((reg = VIC2REG(EP93XX_VIC_IRQStatus) & vic2_intr_enabled)) {
		return ffs(reg) - 1 + 32;
	}

	return -1;
}

void
ep93xx_intr_init(void)
{

	vic1_intr_enabled = 0;
	vic2_intr_enabled = 0;

	/* All interrupts should use IRQ not FIQ */
	VIC1REG(EP93XX_VIC_IntSelect) = 0;
	VIC2REG(EP93XX_VIC_IntSelect) = 0;

	/* Enable IRQs (don't yet use FIQs). */
	enable_interrupts(I32_bit);
}
