/*-
 * Copyright (c) 2005 Olivier Houchard
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");


#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/module.h>
#include <sys/bus.h>
#include <sys/timetc.h>

#include <machine/bus.h>

#include <arm/ep93xx/ep93xxreg.h>
#include <arm/ep93xx/ep93xxvar.h>

/*
 * Driver for the EP930X 32 bits timer.
 */

static unsigned eptimer_get_timecount(struct timecounter *tc);

static struct timecounter eptimer_timecounter = {
	eptimer_get_timecount,	/* get_timecount */
	NULL,			/* no poll_pps */
	~0u,			/* counter_mask */
	508000,			/* frequency */
	"ep timer",		/* name */
	100			/* quality */
};

struct eptimer_softc {
	device_t dev;
	bus_space_tag_t sc_iot;
	bus_space_handle_t sc_ioh;
};

static int
eptimer_probe(device_t dev)
{
	return (0);
}

static int
eptimer_attach(device_t dev)
{
	struct eptimer_softc *sc = device_get_softc(dev);

	sc->dev = dev;
	sc->sc_iot = &ep93xx_bs_tag;
	sc->sc_ioh = EP93XX_APB_HWBASE + EP93XX_APB_Timer3;
	bus_space_write_4(sc->sc_iot, sc->sc_ioh, EP93XX_Timer_Load, 
	    0xffffffff);
	/* Enable the timer, sets it to free running mode @ 508KHz */
	bus_space_write_4(sc->sc_iot, sc->sc_ioh, EP93XX_Timer_Control,
	    (bus_space_read_4(sc->sc_iot, sc->sc_ioh, EP93XX_Timer_Control) |
	    EP93XX_Timer_508CLK | EP93XX_Timer_Enable) &~ EP93XX_Timer_Mode);

	eptimer_timecounter.tc_priv = sc;
	tc_init(&eptimer_timecounter);
	return (0);
}

static unsigned
eptimer_get_timecount(struct timecounter *tc)
{
	struct eptimer_softc *sc = tc->tc_priv;
	
	return (0xffffffff -
	    bus_space_read_4(sc->sc_iot, sc->sc_ioh, EP93XX_Timer_Value));
}

static device_method_t eptimer_methods[] = {
	DEVMETHOD(device_probe, eptimer_probe),
	DEVMETHOD(device_attach, eptimer_attach),
	{0, 0},
};

static driver_t eptimer_driver = {
	"eptimer",
	eptimer_methods,
	sizeof(struct eptimer_softc),
};
static devclass_t eptimer_devclass;

DRIVER_MODULE(eptimer, apb, eptimer_driver, eptimer_devclass, 0, 0);

