/*	$NetBSD: s3c2410.c,v 1.4 2003/08/27 03:46:05 bsh Exp $ */

/*
 * Copyright (c) 2003  Genetec corporation.  All rights reserved.
 * Written by Hiroyuki Bessho for Genetec corporation.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of Genetec corporation may not be used to endorse
 *    or promote products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY GENETEC CORP. ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GENETEC CORP.
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/reboot.h>
#include <sys/module.h>
#include <sys/bus.h>

#include <machine/cpu.h>
#include <machine/bus.h>

#include <machine/cpufunc.h>
#include <machine/intr.h>
#include <arm/s3c2xx0/s3c2410reg.h>
#include <arm/s3c2xx0/s3c2410var.h>
#include <sys/rman.h>

#define S3C2XX0_XTAL_CLK 12000000

#define IPL_LEVELS 13
u_int irqmasks[IPL_LEVELS];

/* prototypes */
static int	s3c2410_probe(device_t);
static void	s3c2410_attach(device_t);
static void	s3c2410_identify(driver_t *, device_t);

static struct resource *s3c2410_alloc_resource(device_t, device_t, int, int *,
        u_long, u_long, u_long, u_int);

static int s3c2410_activate_resource(device_t, device_t, int, int,
        struct resource *);
static int s3c2410_setup_intr(device_t, device_t, struct resource *, int,
        driver_intr_t *, void *, void **);

static device_method_t s3c2410_methods[] = {
	DEVMETHOD(device_probe, s3c2410_probe),
	DEVMETHOD(device_attach, s3c2410_attach),
	DEVMETHOD(device_identify, s3c2410_identify),
	DEVMETHOD(bus_alloc_resource, s3c2410_alloc_resource),
	DEVMETHOD(bus_activate_resource, s3c2410_activate_resource),
	DEVMETHOD(bus_setup_intr, s3c2410_setup_intr),
	{0, 0},
};

static driver_t s3c2410_driver = {
	"s3c2410",
	s3c2410_methods,
	sizeof(struct s3c24x0_softc),
};
static devclass_t s3c2410_devclass;

DRIVER_MODULE(s3c2410, nexus, s3c2410_driver, s3c2410_devclass, 0, 0);

static int
s3c2410_setup_intr(device_t dev, device_t child,
        struct resource *ires,  int flags, driver_intr_t *intr, void *arg,
	    void **cookiep)
{
	return (0);
}

static struct resource *
s3c2410_alloc_resource(device_t bus, device_t child, int type, int *rid,
        u_long start, u_long end, u_long count, u_int flags)
{
	return (NULL);
}

void
s3c2410_identify(driver_t *driver, device_t parent)
{
	
	BUS_ADD_CHILD(parent, 0, "s3c2410", 0);
}

static int
s3c2410_activate_resource(device_t bus, device_t child, int type, int rid,
        struct resource *r)
{
	return (0);
}

int
s3c2410_probe(device_t dev)
{
	return 0;
}

struct s3c2xx0_softc *s3c2xx0_softc = NULL;

void
s3c2410_attach(device_t dev)
{
	struct s3c24x0_softc *sc = device_get_softc(dev);
	bus_space_tag_t iot;

	s3c2xx0_softc = &(sc->sc_sx);
	sc->sc_sx.sc_iot = iot = &s3c2xx0_bs_tag;

	if (bus_space_map(iot,
		S3C2410_INTCTL_BASE, S3C2410_INTCTL_SIZE,
		BUS_SPACE_MAP_LINEAR, &sc->sc_sx.sc_intctl_ioh))
		panic("Cannot map registers");

	/* Map the GPIO registers */
	if (bus_space_map(iot, S3C2410_GPIO_BASE, S3C2410_GPIO_SIZE,
		0, &sc->sc_sx.sc_gpio_ioh))
		panic("Cannot map GPIO registers");
	/* Memory controller */
	if (bus_space_map(iot, S3C2410_MEMCTL_BASE,
		S3C24X0_MEMCTL_SIZE, 0, &sc->sc_sx.sc_memctl_ioh))
		panic("cannot map MEMC");
	/* Clock manager */
	if (bus_space_map(iot, S3C2410_CLKMAN_BASE,
		S3C24X0_CLKMAN_SIZE, 0, &sc->sc_sx.sc_clkman_ioh))
		panic("cannot map clock");

#if 0
	/* Real time clock */
	if (bus_space_map(iot, S3C2410_RTC_BASE,
		S3C24X0_RTC_SIZE, 0, &sc->sc_sx.sc_rtc_ioh))
		FAIL("RTC");
#endif

	if (bus_space_map(iot, S3C2410_TIMER_BASE,
		S3C24X0_TIMER_SIZE, 0, &sc->sc_timer_ioh))
		panic("cannot map TIMER");

#if 0
	/* calculate current clock frequency */
	s3c24x0_clock_freq(&sc->sc_sx);
	printf(": fclk %d MHz hclk %d MHz pclk %d MHz",
	       sc->sc_sx.sc_fclk / 1000000, sc->sc_sx.sc_hclk / 1000000,
	       sc->sc_sx.sc_pclk / 1000000);
	printf("\n");
#endif

	/* get busdma tag for the platform */
}

/*
 * fill sc_pclk, sc_hclk, sc_fclk from values of clock controller register.
 *
 * s3c24x0_clock_freq2() is meant to be called from kernel startup routines.
 * s3c24x0_clock_freq() is for after kernel initialization is done.
 */
void
s3c24x0_clock_freq2(vm_offset_t clkman_base, int *fclk, int *hclk, int *pclk)
{
	uint32_t pllcon, divn;
	int mdiv, pdiv, sdiv;
	int f, h, p;

	pllcon = *(volatile uint32_t *)(clkman_base + CLKMAN_MPLLCON);
	divn = *(volatile uint32_t *)(clkman_base + CLKMAN_CLKDIVN);

	mdiv = (pllcon & PLLCON_MDIV_MASK) >> PLLCON_MDIV_SHIFT;
	pdiv = (pllcon & PLLCON_PDIV_MASK) >> PLLCON_PDIV_SHIFT;
	sdiv = (pllcon & PLLCON_SDIV_MASK) >> PLLCON_SDIV_SHIFT;

	f = ((mdiv + 8) * S3C2XX0_XTAL_CLK) / ((pdiv + 2) * (1 << sdiv));
	h = f;
	if (divn & CLKDIVN_HDIVN)
		h /= 2;
	p = h;
	if (divn & CLKDIVN_PDIVN)
		p /= 2;

	if (fclk) *fclk = f;
	if (hclk) *hclk = h;
	if (pclk) *pclk = p;

}

void
s3c24x0_clock_freq(struct s3c2xx0_softc *sc)
{
	vm_offset_t va;
	
	va = sc->sc_clkman_ioh;
	s3c24x0_clock_freq2(va, &sc->sc_fclk, &sc->sc_hclk, &sc->sc_pclk);
}

void
cpu_reset(void)
{
	(void) disable_interrupts(I32_bit|F32_bit);

	*(volatile unsigned int *)(S3C2410_WDT_BASE + WDT_WTCON)
		= (0 << WTCON_PRESCALE_SHIFT) | WTCON_ENABLE |
		WTCON_CLKSEL_16 | WTCON_ENRST;
	for(;;);
}

int
arm_get_next_irq(void)
{
	return (-1);
}

void
arm_mask_irq(uintptr_t irq)
{
}

void
arm_unmask_irq(uintptr_t irq)
{
}
