#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/param.h>

#include "emac.h"
#include "lib.h"
#include "ee.h"
#include "board.h"

extern unsigned char mac[];

#define TSC_FPGA
#define XMODEM_DL

#ifdef TSC_FPGA
#include "at91rm9200.h"
#include "spi_flash.h"
#include "fpga.h"

#define FPGA_OFFSET  (15 * FLASH_PAGE_SIZE)
#define FPGA_LEN     (212608)

char buffer[FPGA_LEN];

const struct fpga main_fpga = 
{
    AT91C_BASE_PIOB, AT91C_PIO_PB0,
    AT91C_BASE_PIOC, AT91C_PIO_PC11,
    AT91C_BASE_PIOB, AT91C_PIO_PB2,
    AT91C_BASE_PIOC, AT91C_PIO_PC12
};

static void
fpga_load(void)
{
	int len, off, i, offset;
	char *addr = buffer;

	len = FPGA_LEN;
	offset = FPGA_OFFSET;
	for (i = 0; i < len; i+= FLASH_PAGE_SIZE) {
		off = i + offset;
		SPI_ReadFlash(off, addr + i, FLASH_PAGE_SIZE);
	}
	fpga_init(&main_fpga);
	fpga_clear(&main_fpga);
	fpga_write_bytes(&main_fpga, addr, len);
	fpga_done(&main_fpga);
}
#endif

static void
MacFromEE()
{	
	uint32_t sig;
#if 0
	uint8_t euid64[8] = { 0x00, 0x30, 0x96, 0x20,
			      0x00, 0x00, 0x00, 0x07 };
#endif

#if 0
	printf("writing...\n");
	sig = 0xaa55aa55;
	EEWrite(0, (uint8_t *)&sig, sizeof(sig));
	EEWrite(48, euid64, sizeof(euid64));
#endif
	sig = 0;
	EERead(0, (uint8_t *)&sig, sizeof(sig));
	if (sig != 0xaa55aa55)
		return;
	EERead(48, mac, 3);
	EERead(48+5, mac+3, 3);
	printf("MAC %x:%x:%x:%x:%x:%x\n", mac[0],
	  mac[1], mac[2], mac[3], mac[4], mac[5]);
}

#ifdef XMODEM_DL
#define FLASH_OFFSET (0 * FLASH_PAGE_SIZE)
#define FPGA_OFFSET  (15 * FLASH_PAGE_SIZE)
#define FPGA_LEN     (212608)
#define KERNEL_OFFSET (220 * FLASH_PAGE_SIZE)
#define KERNEL_LEN (6 * 1024 * FLASH_PAGE_SIZE)

static void
UpdateFlash(int offset)
{
	char *addr = (char *)0x20000000 + (1 << 20); /* Load to base + 1MB */
	int len, i, off;

	while ((len = xmodem_rx(addr)) == -1)
		continue;
	printf("\nDownloaded %u bytes.\n", len);
	for (i = 0; i < len; i+= FLASH_PAGE_SIZE) {
		off = i + offset;
		SPI_WriteFlash(off, addr + i, FLASH_PAGE_SIZE);
	}
}
void
Update(void)
{
	UpdateFlash(FLASH_OFFSET);
}

#else
void
Update(void)
{
}
#endif

void
board_init(void)
{
#ifdef TSC_FPGA
    SPI_InitFlash();
    fpga_load();
#endif
    EEInit();
    MacFromEE();
}

#include "../bootspi/ee.c"
