/*-
 * Copyright (c) 2006 M. Warner Losh.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $FreeBSD: src/sys/boot/arm/at91/libat91/lib.h,v 1.1 2006/04/19 17:16:49 imp Exp $
 */

#include "at91rm9200.h"
#include "fpga.h"
#include "lib.h"

void
fpga_init(const struct fpga *fpga)
{
	fpga->fpga_prog_n_pio->PIO_PER = fpga->fpga_prog_n_mask;
	fpga->fpga_prog_n_pio->PIO_OER = fpga->fpga_prog_n_mask;

	fpga->fpga_cclk_pio->PIO_PER = fpga->fpga_cclk_mask;
	fpga->fpga_cclk_pio->PIO_OER = fpga->fpga_cclk_mask;

	fpga->fpga_din_pio->PIO_PER = fpga->fpga_din_mask;
	fpga->fpga_din_pio->PIO_OER = fpga->fpga_din_mask;

	fpga->fpga_done_pio->PIO_PER = fpga->fpga_done_mask;
	fpga->fpga_done_pio->PIO_ODR = fpga->fpga_done_mask;
}

void
fpga_clear(const struct fpga *fpga)
{
	fpga->fpga_prog_n_pio->PIO_SODR = fpga->fpga_prog_n_mask;
	Delay(1000);
	fpga->fpga_prog_n_pio->PIO_CODR = fpga->fpga_prog_n_mask;
	Delay(1);
	fpga->fpga_prog_n_pio->PIO_SODR = fpga->fpga_prog_n_mask;
	Delay(4000);
}

void
fpga_write_bytes(const struct fpga *fpga, const unsigned char *buf, int len)
{
	int i, j;
	unsigned byte;

	for (j = 0; j < len; j++) {
		byte = buf[j];
		for (i = 7; i >= 0; i--) {
			fpga->fpga_cclk_pio->PIO_CODR = fpga->fpga_cclk_mask;
			if (byte & (1 << i))
				fpga->fpga_din_pio->PIO_SODR = fpga->fpga_din_mask;
			else
				fpga->fpga_din_pio->PIO_CODR = fpga->fpga_din_mask;
			// 10ns delay needed?
			fpga->fpga_cclk_pio->PIO_SODR = fpga->fpga_cclk_mask;
		}
	}
}

void
fpga_done(const struct fpga *fpga)
{
	while (!(fpga->fpga_done_pio->PIO_PSR & fpga->fpga_done_mask))
		continue;
}
