/*******************************************************************************
 *
 * Filename: irda.c
 *
 * IrDA functions - development status only!
 *
 * Revision information:
 *
 * 20JAN2005	kb_admin	initial creation - includes Atmel lib functions
 *
 * BEGIN_KBDD_BLOCK
 * No warranty, expressed or implied, is included with this software.  It is
 * provided "AS IS" and no warranty of any kind including statutory or aspects
 * relating to merchantability or fitness for any purpose is provided.  All
 * intellectual property rights of others is maintained with the respective
 * owners.  This software is not copyrighted and is intended for reference
 * only.
 * END_BLOCK
 ******************************************************************************/

#include "AT91RM9200.h"

#include "debug_io.h"
#include "p_string.h"
#include "prompt.h"


#define MAX_FAILURES	32

// required for IrDA test
// Standard IRDA mode
#define AT91C_US_ASYNC_IRDA_MODE (AT91C_US_USMODE_IRDA + AT91C_US_NBSTOP_1_BIT + AT91C_US_PAR_NONE + AT91C_US_CHRL_8_BITS + AT91C_US_CLKS_CLOCK)


#define	MAX_IRDA_RX_BYTES	100


/* Master clock frequency at power-up */
#ifdef USE_80P_20M_CLOCKS
#define AT91C_MASTER_CLOCK 20000000
#else
#define AT91C_MASTER_CLOCK 60000000
#endif

char			irdaRxBuffer[MAX_IRDA_RX_BYTES];
parse_function_t	irda_function;


__inline void AT91F_US_SetIrdaFilter (
	AT91PS_USART pUSART,
	unsigned char value
)
{
	pUSART->US_IF = value;
}

__inline void AT91F_US_DisableRx (
	AT91PS_USART pUSART)     // \arg pointer to a USART controller
{
    // Disable receiver
    pUSART->US_CR = AT91C_US_RXDIS;
}

__inline void AT91F_US_DisableTx (
	AT91PS_USART pUSART)     // \arg pointer to a USART controller
{
    // Disable transmitter
    pUSART->US_CR = AT91C_US_TXDIS;
}

__inline void AT91F_US_EnableRx (
	AT91PS_USART pUSART)     // \arg pointer to a USART controller
{
    // Enable receiver
    pUSART->US_CR = AT91C_US_RXEN;
}

__inline void AT91F_US_EnableTx (
	AT91PS_USART pUSART)     // \arg pointer to a USART controller
{
    // Enable  transmitter
    pUSART->US_CR = AT91C_US_TXEN;
}

__inline void AT91F_PDC_SetNextRx (
	AT91PS_PDC pPDC,     // \arg pointer to a PDC controller
	char *address,       // \arg address to the next bloc to be received
	unsigned int bytes)  // \arg number of bytes to be received
{
	pPDC->PDC_RNPR = (unsigned int) address;
	pPDC->PDC_RNCR = bytes;
}

__inline void AT91F_PDC_SetNextTx (
	AT91PS_PDC pPDC,       // \arg pointer to a PDC controller
	char *address,         // \arg address to the next bloc to be transmitted
	unsigned int bytes)    // \arg number of bytes to be transmitted
{
	pPDC->PDC_TNPR = (unsigned int) address;
	pPDC->PDC_TNCR = bytes;
}

__inline void AT91F_PDC_SetRx (
	AT91PS_PDC pPDC,       // \arg pointer to a PDC controller
	char *address,         // \arg address to the next bloc to be received
	unsigned int bytes)    // \arg number of bytes to be received
{
	pPDC->PDC_RPR = (unsigned int) address;
	pPDC->PDC_RCR = bytes;
}

__inline void AT91F_PDC_SetTx (
	AT91PS_PDC pPDC,       // \arg pointer to a PDC controller
	char *address,         // \arg address to the next bloc to be transmitted
	unsigned int bytes)    // \arg number of bytes to be transmitted
{
	pPDC->PDC_TPR = (unsigned int) address;
	pPDC->PDC_TCR = bytes;
}

__inline void AT91F_PDC_EnableTx (
	AT91PS_PDC pPDC )       // \arg pointer to a PDC controller
{
	pPDC->PDC_PTCR = AT91C_PDC_TXTEN;
}

__inline void AT91F_PDC_EnableRx (
	AT91PS_PDC pPDC )       // \arg pointer to a PDC controller
{
	pPDC->PDC_PTCR = AT91C_PDC_RXTEN;
}

__inline void AT91F_PDC_DisableTx (
	AT91PS_PDC pPDC )       // \arg pointer to a PDC controller
{
	pPDC->PDC_PTCR = AT91C_PDC_TXTDIS;
}

__inline void AT91F_PDC_DisableRx (
	AT91PS_PDC pPDC )       // \arg pointer to a PDC controller
{
	pPDC->PDC_PTCR = AT91C_PDC_RXTDIS;
}

__inline int AT91F_PDC_IsTxEmpty ( // \return return 1 if transfer is complete
	AT91PS_PDC pPDC )       // \arg pointer to a PDC controller
{
	return !(pPDC->PDC_TCR);
}

__inline int AT91F_PDC_IsNextTxEmpty ( // \return return 1 if transfer is complete
	AT91PS_PDC pPDC )       // \arg pointer to a PDC controller
{
	return !(pPDC->PDC_TNCR);
}

__inline void AT91F_PDC_Open (
	AT91PS_PDC pPDC)       // \arg pointer to a PDC controller
{
    // Disable the RX and TX PDC transfer requests
	AT91F_PDC_DisableRx(pPDC);
	AT91F_PDC_DisableTx(pPDC);

	// Reset all Counter register Next buffer first
	AT91F_PDC_SetNextTx(pPDC, (char *) 0, 0);
	AT91F_PDC_SetNextRx(pPDC, (char *) 0, 0);
	AT91F_PDC_SetTx(pPDC, (char *) 0, 0);
	AT91F_PDC_SetRx(pPDC, (char *) 0, 0);

    // Enable the RX and TX PDC transfer requests
	AT91F_PDC_EnableRx(pPDC);
	AT91F_PDC_EnableTx(pPDC);
}

__inline unsigned int AT91F_US_Baudrate (
	const unsigned int main_clock, // \arg peripheral clock
	const unsigned int baud_rate)  // \arg UART baudrate
{
	unsigned int baud_value = ((main_clock*10)/(baud_rate * 16));
	if ((baud_value % 10) >= 5)
		baud_value = (baud_value / 10) + 1;
	else
		baud_value /= 10;
	return baud_value;
}

__inline void AT91F_US_SetBaudrate (
	AT91PS_USART pUSART,    // \arg pointer to a USART controller
	unsigned int mainClock, // \arg peripheral clock
	unsigned int speed)     // \arg UART baudrate
{
	// Define the baud rate divisor register
	pUSART->US_BRGR = AT91F_US_Baudrate(mainClock, speed);
}

__inline void AT91F_US_SetTimeguard (
	AT91PS_USART pUSART,    // \arg pointer to a USART controller
	unsigned int timeguard) // \arg timeguard value
{
	// Write the Timeguard Register
	pUSART->US_TTGR = timeguard ;
}

__inline unsigned int AT91F_US_TxReady (
	AT91PS_USART pUSART )     // \arg pointer to a USART controller
{
    return (pUSART->US_CSR & AT91C_US_TXRDY);
}

__inline void AT91F_US_PutChar (
	AT91PS_USART pUSART,
	int character )
{
    pUSART->US_THR = (character & 0x1FF);
}

__inline unsigned int AT91F_US_RxReady (
	AT91PS_USART pUSART )     // \arg pointer to a USART controller
{
    return (pUSART->US_CSR & AT91C_US_RXRDY);
}

__inline int AT91F_US_GetChar (
	const AT91PS_USART pUSART)
{
    return((pUSART->US_RHR) & 0x1FF);
}

__inline void AT91F_US1_CfgPMC (void)
{
	AT91F_PMC_EnablePeriphClock(
		AT91C_BASE_PMC, // PIO controller base address
		((unsigned int) 1 << AT91C_ID_US1));
}

__inline void AT91F_US1_CfgPIO (void)
{
	// Configure PIO controllers to periph mode
	AT91F_PIO_CfgPeriph(
		AT91C_BASE_PIOB, // PIO controller base address
		((unsigned int) AT91C_PB21_RXD1    ) |
		((unsigned int) AT91C_PB26_RTS1    ) |
		((unsigned int) AT91C_PB25_DSR1    ) |
		((unsigned int) AT91C_PB24_CTS1    ) |
		((unsigned int) AT91C_PB19_DTR1    ) |
		((unsigned int) AT91C_PB23_DCD1    ) |
		((unsigned int) AT91C_PB20_TXD1    ) |
		((unsigned int) AT91C_PB18_RI1     ), // Peripheral A
		0); // Peripheral B
}

__inline void AT91F_US_Configure (
	AT91PS_USART pUSART,     // \arg pointer to a USART controller
	unsigned int mainClock,  // \arg peripheral clock
	unsigned int mode ,      // \arg mode Register to be programmed
	unsigned int baudRate ,  // \arg baudrate to be programmed
	unsigned int timeguard ) // \arg timeguard to be programmed
{
    // Disable interrupts
    pUSART->US_IDR = (unsigned int) -1;

    // Reset receiver and transmitter
    pUSART->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX | AT91C_US_RXDIS | AT91C_US_TXDIS ;

	// Define the baud rate divisor register
	AT91F_US_SetBaudrate(pUSART, mainClock, baudRate);

	// Write the Timeguard Register
	AT91F_US_SetTimeguard(pUSART, timeguard);

    // Clear Transmit and Receive Counters
    AT91F_PDC_Open((AT91PS_PDC) &(pUSART->US_RPR));

    // Define the USART mode
    pUSART->US_MR = mode  ;

}


/*
 * void AT91F_IRDA_putc(char)
 * return non-zero on success
 */

void AT91F_IRDA_putc(char buffer) {

	while (!AT91F_US_TxReady((AT91PS_USART)AT91C_BASE_US1));
	AT91F_US_PutChar((AT91PS_USART)AT91C_BASE_US1, buffer);
}


/*
 * char AT91F_IRDA_getc(void)
 * return non-zero on success
 */

int AT91F_IRDA_getc(char *cPtr) {

	if (AT91F_US_RxReady((AT91PS_USART)AT91C_BASE_US1)) {
		*cPtr = (char)AT91F_US_GetChar((AT91PS_USART)AT91C_BASE_US1);
		return (1);
	}
	return (0);
}


/*
 * void IrDAPrint(char *cPtr)
 */
void IrDAPrint(char *cPtr) {
	while (*cPtr)
		AT91F_IRDA_putc(*cPtr++);
}


/*
 * int ReceivedKeyString(int)
 */

int ReceivedKeyString(int charCount) {

	char	*cPtr;

	for (cPtr = irdaRxBuffer; charCount--; cPtr++)
		if (!(p_strcmp("Simple IRDA test response", cPtr)))
			return (1);
	return (0);
}


/*
 * void IrDA_Test(void)
 */

void IrDA_Ping(void) {

	int	thisSecs = 0, charCount, baudRate = 9600;
	char	*cPtr, c;
	int	timeout = 10;

	for (cPtr = irdaRxBuffer, charCount = 0; charCount < MAX_IRDA_RX_BYTES;
		++charCount)
		*cPtr++ = 0;

	// Open PIO & PMC for USART1
	AT91F_US1_CfgPIO();
	AT91F_US1_CfgPMC();

	// Configure USART1
	AT91F_US_Configure (
		AT91C_BASE_US1,
		AT91C_MASTER_CLOCK,
		AT91C_US_ASYNC_IRDA_MODE,
		baudRate,
		0);

	// Configure filter (!!!even in transmision mode!!!)
	AT91F_US_SetIrdaFilter(AT91C_BASE_US1, 10);

	// Disable pullup
	//AT91C_BASE_PIOB->PIO_PPUDR = AT91C_PB20_TXD1 | AT91C_PB21_RXD1;


	// Receive and send 1 character using RHR & THR
	// ============= IRDA is HALF-DUPLEX: Reception mode ==
	// Enable reception

	// send key string several times
	DebugPrint("Sending key string\n\r");

	AT91F_US_DisableRx(AT91C_BASE_US1);
	AT91F_US_EnableTx(AT91C_BASE_US1);
	IrDAPrint("IrDA ping test\0\0");
	IrDAPrint("IrDA ping test\0\0");
	IrDAPrint("IrDA ping test\0\0");
	IrDAPrint("IrDA ping test\0\0");

	// wait for key string received
	while ( !(AT91C_BASE_US1->US_CSR & AT91C_US_TXEMPTY) );
	AT91F_US_DisableTx(AT91C_BASE_US1);
	AT91F_US_EnableRx(AT91C_BASE_US1);

	DebugPrint("Waiting for return key string\n\r");

	cPtr = irdaRxBuffer;
	charCount = 0;
	while (timeout) {

		if (thisSecs != GetSeconds()) {
			thisSecs = GetSeconds();
			DebugPrint("Waiting for return key string\n\r");
			--timeout;
		}

		if (AT91F_IRDA_getc(cPtr)) {
			DebugPutc(*cPtr);
			++cPtr;
			++charCount;
			if (charCount >= MAX_IRDA_RX_BYTES) {
				cPtr = irdaRxBuffer;
				charCount = 0;
			}

			if (ReceivedKeyString(charCount))
				break;
		}
	}

	DebugPrint("IrDA test ");

	if (!timeout) {
		DebugPrint("FAIL\n\r");
	} else {
		DebugPrint("Key string received\n\r");
		DebugPrint("PASS\n\r");
	}
}


/*
 * .KB_C_FN_DEFINITION_START
 * int irda_function_parse(int argc, char *argv[])
 *  This global function parses text from the command line.
 * .KB_C_FN_DEFINITION_END
 */
int irda_function_parse(int argc, char *argv[]) {

	IrDA_Ping();
	return (0);
}


/*
 * .KB_C_FN_DEFINITION_START
 * void irda_function_help(void)
 *  This global displays help info for the corresponding command.
 * .KB_C_FN_DEFINITION_END
 */
void irda_function_help(void) {
	DebugPrint("\tIrDA ping test\n\r");
	DebugPrint("\t\tirda\n\r");
}


/*
 * .KB_C_FN_DEFINITION_START
 * void InitProcessor(void)
 *  This global function initializes the generic support functions.
 * .KB_C_FN_DEFINITION_END
 */
void InitIrDA(void) {
	irda_function.f_string = "irda";
	irda_function.parse_function = irda_function_parse;
	irda_function.help_function = irda_function_help;
	RegisterFunction(&irda_function);
}
