/*-
 * Copyright (C) 2006 M. Warner Losh. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#ifndef _SYS_GPIO_H
#define _SYS_GPIO_H

#ifndef _KERNEL
#include <sys/types.h>
#endif
#include <sys/ioccom.h>

struct gpio_info
{
	uint32_t	output_status;	/* Current state of output pins */
	uint32_t	input_status;	/* 1->in 0->out bitmask */
	uint32_t	highz_status;	/* 1->highz 0->driven bitmask */
	uint32_t	pullup_status;	/* 1->pullup engaged 0->floating */
	uint32_t	glitch_status;	/* 0-> no glitch filter 1->gf */
	uint32_t	enabled_status;	/* 1->used for gpio 0->other */
	uint32_t	periph_status;	/* 0->A periph 1->B periph */
	uint32_t	intr_status;	/* 1-> ISR enabled, 0->disabled */
	uint32_t	extra_status[8];/* Extra status info, device depend */
};

struct gpio_cfg
{
	uint32_t	cfgmask; 	/* which things change */
#define	GPIO_CFG_INPUT		1	/* configure input/output pins */
#define	GPIO_CFG_HI_Z		2	/* HiZ */
#define	GPIO_CFG_PULLUP		4	/* Enable/disable pullup resistors */
#define GPIO_CFG_GLITCH		8	/* Glitch filtering */
#define GPIO_CFG_ENABLED	0x10	/* Use pin for GPIO or other */
#define	GPIO_CFG_PERIPH		0x20	/* Select which peripheral to use */
#define	GPIO_CFG_ISR		0x40	/* Select pin for ISR */
	uint32_t	iomask;		/* Mask of bits to change */
	uint32_t	input;		/* or output */
	uint32_t	hi_z;		/* Disable output */
	uint32_t	pullup;		/* Enable pullup resistor */
	uint32_t	glitch;		/* Glitch filtering */
	uint32_t	enabled;	/* Enabled for GPIO (1) or other (0) */
	uint32_t	periph;		/* Select which periph, if possible */
};

#define GPIO_SET	_IOW('g', 0, uint32_t)	/* Turn bits on */
#define GPIO_CLR	_IOW('g', 1, uint32_t)	/* Turn bits off */
#define GPIO_READ	_IOR('g', 2, uint32_t)	/* Read input bit state */
#define GPIO_INFO	_IOR('g', 3, struct gpio_info)	/* State of gpio cfg */
#define GPIO_CFG	_IOW('g', 4, struct gpio_cfg)	/* Configure gpio */

#endif /* _SYS_GPIO_H */
